function [M,t] = NMRexp(data)
% [M,T] = NMRexp(DATA)
%
% NMR experiment for a small volume composed of nuclei. We compute the 
% trajectory of the bulk magnetization in response of an excitation B1 and
% in a static field B0.
%
% DATA  is a structure carrying the parameters of experiment.
%       data.B0     static magnetic field magnitude
%       data.gamma  gyroscopic ratio
%       data.B1     excitation field magnitude
%       data.omega  excitation frequency
%       data.M0     initial magnetization (vector)
%       data.Texc   duration of excitation
%       data.Theta  flip angle in radian (fixes the duration of excitation)
%       data.Nt     number of time samples
%
% M     is a matrix containing the 3 component of the magnetization through
%       time.
%
% T     is the time vector (in seconds)
%
% (c) Matthieu Guerquin-Kern, 2007.10.21

%% Preliminar calculations
tic;

% experiment parameters
if nargin==0
    data.null = 0;
end
disp('%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%');
disp('Parameters of the experiment');
disp('%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%');
if isfield(data,'B0');
    B0 = data.B0;
else
    B0 = 18.3;          % T
end
disp(sprintf('B0       = %2.3f T',B0));

if isfield(data,'gamma');
    gamma = data.gamma;
else
    gamma = 2.678*1e8;  % rad.T^-1.s^-1
end
disp(sprintf('gamma    = %1.3e rad.T^-1.s^-1',gamma));

omega0 = gamma*B0;      % rad.s^-1
disp(sprintf('f0       = %3.3f MHz',abs(omega0/2/pi*1e-6)));

if isfield(data,'B1');
    B1 = data.B1;
else
    B1 = 1;             % T
end
disp(sprintf('B1       = %2.3f T',B1));

omega1 = -gamma*B1;     % rad.s^-1
disp(sprintf('f1       = %3.3f MHz',abs(omega1/2/pi*1e-6)));

if isfield(data,'omega');
    omega = data.omega;
else
    omega = omega0;     % rad.s^-1
end
disp(sprintf('omega_RF = %3.3f MHz',abs(omega/2/pi*1e-6)));

if isfield(data,'M0');
    M0 = data.M0;
else
    M0 = [0;0;1];   	% J.T^-1
end
disp(sprintf('M0       = %1.3f J.T^-1',norm(M0)));

if isfield(data,'Theta');
    Theta = data.Theta;
else
    if isfield(data,'Texc');
        Theta = data.Texc*omega1;
    end
    Theta = pi/2;   	% rad
end
Texc = abs(Theta/omega1);
disp(sprintf('duration of excitation = %2.3f ps',1e9*Texc));
disp(sprintf('flip angle = %2.3f rad',Theta));

Domega = omega-omega0;
omega_eff = sqrt(Domega^2+omega1^2);
disp(sprintf('f_eff    = %3.3f MHz',abs(omega_eff/2/pi*1e-6)));
disp(sprintf('B_eff    = %3.3f T',abs(omega_eff)/gamma));
theta = acos(Domega/omega_eff);
disp(sprintf('theta_eff    = %1.3f rad',theta));

if isfield(data,'Trel');
    Trel = data.Trel;
else
    Trel = 0;           % s
end
disp(sprintf('duration of relaxation = %2.3f ps',1e9*Trel));

if isfield(data,'Nt');
    Nt = data.Nt;
else
    Nt = 5e2;
end
disp(sprintf('time samples = %d',Nt));

% time vector
Ttot = (Texc+Trel);
Dt = Ttot/Nt;          % s
t = 0:Dt:Ttot-Dt;      % s
Nexc = round(Texc/Ttot);
Nrel = Nt-Nexc;

% declaration of the magnetization vector
M = zeros(3,Nt);

% Rotation matrices
R0 = @(t) [cos(omega*t) sin(omega*t) 0;-sin(omega*t) cos(omega*t) 0;0 0 1];
R1 = [cos(theta) 0 sin(theta);0 1 0;sin(theta) 0 cos(theta)];
R2 = @(t) [cos(omega_eff*t) sin(omega_eff*t) 0;-sin(omega_eff*t) cos(omega_eff*t) 0;0 0 1];
R0rel = @(t) [cos(omega0*t) sin(omega0*t) 0;-sin(omega0*t) cos(omega0*t) 0;0 0 1];

% Initial magnetization
M0r = (R0(0)*R1*R2(0))^(-1)*M0;

disp('...');

%% Computation

% excitation
for i = 1:Nexc
    M(:,i) = R0(t(i))*R1*R2(t(i))*M0r;
end

% relaxation
Mchange = (R0rel(Texc))^(-1)*M(:,Nexc);
for i = Nexc+1:Ntot
    M(:,i) = R0rel(t(i))*[Mchange(1)*exp((Texc-t(i))/T2);Mchange(2)*exp((Texc-t(i))/T2);norm(M0)+(Mchange(3)-norm(M0))*exp((Texc-t(i))/T1)];
end

disp(sprintf(' Done in %3.3f seconds',toc));