function [M,t] = NMRrel(data)
% [M,T] = NMRrel(DATA)
%
% NMR experiment for a small volume composed of nuclei. We compute the 
% trajectory of the bulk magnetization in a static field B0 and taking T1
% and T2 time constants into account.
%
% DATA  is a structure carrying the parameters of experiment.
%       data.B0     static magnetic field magnitude
%       data.gamma  gyroscopic ratio
%       data.M0     initial magnetization (vector)
%       data.T1     T1 time constant (parallel relaxation)
%       data.T2     T2 time constant (transverse relaxation)
%       data.Trel   duration of relaxation
%       data.Nt     number of time samples
%
% M     is a matrix containing the 3 component of the magnetization through
%       time.
%
% T     is the time vector (in seconds)
%
% (c) Matthieu Guerquin-Kern, 2007.10.21

%% Preliminar calculations
tic;

% experiment parameters
if nargin==0
    data.null = 0;
end
disp('%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%');
disp('Parameters of the relaxation');
disp('%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%');
if isfield(data,'B0');
    B0 = data.B0;
else
    B0 = 18.3;          % T
end
disp(sprintf('B0       = %2.3f T',B0));

if isfield(data,'gamma');
    gamma = data.gamma;
else
    gamma = 2.678*1e8;  % rad.T^-1.s^-1
end
disp(sprintf('gamma    = %1.3e rad.T^-1.s^-1',gamma));

omega0 = gamma*B0;      % rad.s^-1
disp(sprintf('f0       = %3.3f MHz',abs(omega0/2/pi*1e-6)));

if isfield(data,'M0');
    M0 = data.M0;
else
    M0 = [1;0;0];   	% J.T^-1
end
disp(sprintf('M0       = %1.3f J.T^-1',norm(M0)));

if isfield(data,'T1');
    T1 = data.T1;
else
    T1 = 1;           % s
end
disp(sprintf('T1 = %2.3f s',T1));

if isfield(data,'T2');
    T2 = data.T2;
else
    T2 = 0.2;           % s
end
disp(sprintf('T2 = %2.3f s',T2));

if isfield(data,'Trel');
    Trel = data.Trel;
else
    Trel = 3*T1;           % s
end
disp(sprintf('duration = %2.3f s',Trel));

if isfield(data,'Nt');
    Nt = data.Nt;
else
    Nt = 5e2;
end
disp(sprintf('time samples = %d',Nt));

% time vector
Dt = Trel/Nt;          % s
t = 0:Dt:Trel-Dt;      % s

% declaration of the magnetization vector
M = zeros(3,Nt);

% Rotation matrices
R0 = @(t) [cos(omega0*t) sin(omega0*t) 0;-sin(omega0*t) cos(omega0*t) 0;0 0 1];

% Initial magnetization
M0r = (R0(0))^(-1)*M0;

disp('...');

%% Computation

% relaxation
for i = 1:Nt
    M(:,i) = R0(t(i))*[M0r(1)*exp((0-t(i))/T2);M0r(2)*exp((0-t(i))/T2);norm(M0)+(M0r(3)-norm(M0))*exp((0-t(i))/T1)];
end

disp(sprintf(' Done in %3.3f seconds',toc));