function PlotNMRexp(M,t,plot)
% PlotNMRexp(M,T,PLOT)
%
% NMR experiment for a small volume composed of nuclei. We compute the 
% trajectory of the bulk magnetization in response of an excitation B1 and
% in a static field B0.
%
% M     is a matrix containing the 3 component of the magnetization through
%       time.
%
% T     is the time vector (in seconds)
%
% PLOT  is a structure carrying the value we want to display.
%       plot.threeD         to display the 3D track
%       plot.X          to display the magnetization along X axis
%       plot.Y          to display the magnetization along Y axis
%       plot.XY         to display the magnetization component in plane XY
%       plot.Z          to display the magnetization along Z axis
%       plot.save       to save the displayed graphs into pictures (or
%                       video)
%       plot.video      to display the graphs as videos
%
% (c) Matthieu Guerquin-Kern, 2007.10.21

%% Default

if nargin<3
    plot.video = true;
    plot.threeD = true;
end

if ~isfield(plot,'video')
    plot.video = false;
end

if ~isfield(plot,'save')
    plot.save = false;
end

if ~isfield(plot,'threeD')
    plot.threeD = false;
end

if ~isfield(plot,'threeD')
    plot.threeD = false;
end
if ~isfield(plot,'X')
    plot.X = false;
end
if ~isfield(plot,'Y')
    plot.Y = false;
end
if ~isfield(plot,'XY')
    plot.XY = false;
end
if ~isfield(plot,'Z')
    plot.Z = false;
end


%% Plotings
close all;

Nt = length(t);

if plot.video
    if plot.save
        dir = input('Name of the directory for the saved videos and/or images? : ', 's');
        %dir = ['''' dir ''''];
        if ~exist(dir,'dir')
            mkdir(dir);
        end
        cd(dir);
    end
    
    for i = 1:Nt
        if plot.threeD
            clf;
            gcf = figure(1);
            hold on;
            % Axis
            PlotThreeDaxis;
            % Trajectory
            PlotTrajectory(M(:,1:i));
            title([sprintf('time : %2.3f ', 1e9*t(i)), 'ps.'],'fontsize', 15);
            hold off;
            SaveVidPlot(i,'threeD',plot.save);
        end
        if plot.X
            gcf = figure(2);
            range = [t(1) t(end) min(M(1,:)) max(M(1,:))];
            PlotOneDTraj(M(1,1:i),t(1:i),range);
            ylabel('X-component of the magnetization');
            SaveVidPlot(i,'X',plot.save);
        end
        if plot.Y
            gcf = figure(3);
            range = [t(1) t(end) min(M(2,:)) max(M(2,:))];
            PlotOneDTraj(M(2,1:i),t(1:i),range);
            ylabel('Y-component of the magnetization');
            SaveVidPlot(i,'Y',plot.save);
        end
        if plot.Z
            gcf = figure(4);
            range = [t(1) t(end) min(M(3,:)) max(M(3,:))];
            PlotOneDTraj(M(3,1:i),t(1:i),range);
            ylabel('Z-component of the magnetization');
            SaveVidPlot(i,'Z',plot.save);
        end
        if plot.XY
            gcf = figure(5);
            range = [min(M(1,:)) max(M(1,:)) min(M(2,:)) max(M(2,:))];
            PlotTwoDTraj(M(1,1:i),M(2,1:i),range);
            xlabel('X-component of the magnetization');
            ylabel('Y-component of the magnetization');
            SaveVidPlot(i,'XY',plot.save);
        end
    end
    if plot.save
        cd ..;
    end
else
    if plot.save
        dir = input('Name of the directory for the saved images? : ', 's');
        %dir = ['''' pwd '/' dir ''''];
        if ~exist(dir,'dir')
            mkdir(dir);
        end
        cd(dir);
    end
    if plot.threeD
        gcf = figure;
        hold on;
        % Axis
        PlotThreeDaxis;
        % Trajectory
        PlotTrajectory(M);
        %title([sprintf('time : %2.3f ', 1e9*t(i)), 'ps.'],'fontsize', 15);
        hold off;
        SaveImPlot('threeD',plot.save);
    end
    if plot.X
        gcf = figure;
        hold on;
        PlotOneDTraj(M(1,:),t);
        ylabel('X-component of the magnetization');
        hold off;
        SaveImPlot('X',plot.save);
    end
    if plot.Y
        gcf = figure;
        hold on;
        PlotOneDTraj(M(2,:),t);
        ylabel('Y-component of the magnetization');
        hold off;
        SaveImPlot('Y',plot.save);
    end
    if plot.Z
        gcf = figure;
        hold on;
        PlotOneDTraj(M(3,:),t);
        ylabel('Z-component of the magnetization');
        hold off;
        SaveImPlot('Z',plot.save);
    end
    if plot.XY
        gcf = figure;
        hold on;
        PlotTwoDTraj(M(1,:),M(2,:));
        xlabel('X-component of the magnetization');
        ylabel('Y-component of the magnetization');
        hold off;
        SaveImPlot('XY',plot.save);
    end
    if plot.save
        cd ..;
    end
end

function PlotThreeDaxis()
% Axes
axis off;
plot3(0,0,0,'ro','MarkerEdgeColor','r','MarkerFaceColor','r','MarkerSize',10); % center point
plot3([0,1.5],[0,0],[0,0],'k-','LineWidth',3.5); % X-axis
plot3([0,0],[0,1.5],[0,0],'k-','LineWidth',3.5); % Y-axis
plot3([0,0],[0,0],[0,1.5],'k-','LineWidth',3.5); % Z-axis
[sx,sy,sz] = sphere(20);
col = summer;
h = surf(sx,sy,sz,'FaceColor',col(40,:),'FaceAlpha',0.6,'EdgeAlpha',0);

axis([-1 1 -1 1 -1 1]);
axis square;
view(110,10);

function PlotVector(M)
% Curent vector
plot3([0,M(1)],[0,M(2)],[0,M(3)],'r-','LineWidth',2)
plot3(M(1),M(2),M(3),'rd','MarkerEdgeColor','r','MarkerFaceColor','r','MarkerSize',10); % center point

function PlotTrajectory(M)
plot3(M(1,1:end),M(2,1:end),M(3,1:end),'b-','LineWidth',2.5);
PlotVector(M(:,end));

function PlotOneDTraj(M,t,range)
plot(1e9*t,M,'b-','LineWidth',4);
if nargin==3
    range(1:2) = 1e9*range(1:2);
    axis(range);
end
xlabel('time in ps');

function PlotTwoDTraj(Mx,My,range)
plot(Mx,My,'b-','LineWidth',4);
if nargin==3
    axis(range);
end
axis square;

function SaveVidPlot(i,type,test)
drawnow;
if test
    name = sprintf('%s%00u.png',type,i);
    if exist(name,'file')&&i==1
        erase = input(sprintf('The file ''%s'' already exists. Replace it? (Y/N) ',name),'s');
        if lower(erase)=='y'
            print('-dpng','-r100',name);
        else
            cd ..
            error('Saving plot cancelled. Change the saving directory and retry!');
        end
    else
        print('-dpng','-r100',name);
    end
end

function SaveImPlot(type,test)
drawnow;
if test
    name = sprintf('%s.eps',type);
    if exist(name,'file')
        erase = input(sprintf('The file ''%s'' already exists. Replace it? (Y/N) ',name),'s');
        if lower(erase)=='y'
            print('-depsc','-r200',name);
        else
            cd ..
            error('Saving plot cancelled. Change the saving directory and retry!');
        end
    else
        print('-depsc','-r200',name);
    end
end